/*
 * Copyright (c) 2015 - 2016 , Freescale Semiconductor, Inc.
 * Copyright 2016-2017 NXP
 * All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NXP "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NXP OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "device_registers.h"
#include "interrupt_manager.h"
#include "clock_manager.h"
#include "pcc_hal.h"
#include "scg_hal.h"
#include "port_hal.h"
#include "gpio_hal.h"
#include "ftm_hal.h"
#include "lin_driver.h"
#include "lin_common_api.h"
#include "clockMan1.h"
#include "pin_mux.h"
#include "lin1.h"

/*******************************************************************************
 * Definitions
 ******************************************************************************/
/* This example is setup to work by default with EVB. To use it with other boards
   please comment the following line
*/
#define EVB

#ifdef EVB
    #define PORT_LED0_INDEX           (0u)
    #define PORT_LED1_INDEX           (15u)
    #define PORT_LED2_INDEX           (16u)
    #define PORT_LED0_MASK            (0x1u << PORT_LED0_INDEX)
    #define PORT_LED1_MASK            (0x1u << PORT_LED1_INDEX)
    #define PORT_LED2_MASK            (0x1u << PORT_LED2_INDEX)
    #define LED_GPIO_PORT             (PTD)
    #define PORT_BTN0_INDEX           (12u)
    #define PORT_BTN1_INDEX           (13u)
    #define PORT_BTN0_MASK            (0x1u << PORT_BTN0_INDEX)
    #define PORT_BTN1_MASK            (0x1u << PORT_BTN1_INDEX)
    #define BTN_GPIO_PORT             (PTC)
    #define BTN_PORT_NAME             (PORTC)
    #define BTN_PORT_IRQn             (PORTC_IRQn)
    #define USE_LIN_XCVR              (1)
    #define LIN_XCVR_ENABLE_PIN       (9UL)
    #define LIN_XCVR_ENABLE_MASK      (0x1u << LIN_XCVR_ENABLE_PIN)
    #define LIN_XCVR_ENABLE_GPIO_PORT (PTE)
#else
    #define PORT_LED0_INDEX           (0u)
    #define PORT_LED1_INDEX           (1u)
    #define PORT_LED2_INDEX           (2u)
    #define PORT_LED0_MASK            (0x1u << PORT_LED0_INDEX)
    #define PORT_LED1_MASK            (0x1u << PORT_LED1_INDEX)
    #define PORT_LED2_MASK            (0x1u << PORT_LED2_INDEX)
    #define LED_GPIO_PORT             (PTC)
    #define PORT_BTN0_INDEX           (12u)
    #define PORT_BTN1_INDEX           (13u)
    #define PORT_BTN0_MASK            (0x1u << PORT_BTN0_INDEX)
    #define PORT_BTN1_MASK            (0x1u << PORT_BTN1_INDEX)
    #define BTN_GPIO_PORT             (PTC)
    #define BTN_PORT_NAME             (PORTC)
    #define BTN_PORT_IRQn             (PORTC_IRQn)
#endif

#define RGB_RED_OFF        LED_GPIO_PORT->PSOR |= PORT_LED1_MASK
#define RGB_RED_ON         LED_GPIO_PORT->PCOR |= PORT_LED1_MASK
#define RGB_RED_TOGGLE     LED_GPIO_PORT->PTOR |= PORT_LED1_MASK

#define RGB_GREEN_OFF      LED_GPIO_PORT->PSOR |= PORT_LED2_MASK
#define RGB_GREEN_ON       LED_GPIO_PORT->PCOR |= PORT_LED2_MASK
#define RGB_GREEN_TOGGLE   LED_GPIO_PORT->PTOR |= PORT_LED2_MASK

#define RGB_BLUE_OFF       LED_GPIO_PORT->PSOR |= PORT_LED0_MASK
#define RGB_BLUE_ON        LED_GPIO_PORT->PCOR |= PORT_LED0_MASK
#define RGB_BLUE_TOGGLE    LED_GPIO_PORT->PTOR |= PORT_LED0_MASK

/* (CLK (Mhz)* timer period (us) / Prescaler) */
#define TIMER_VALUE (uint16_t)(48u*500u)
#define TIMER_1US   (uint16_t)(48u)

#define MOTOR_SELECTION_INCREASE (1u)
#define MOTOR_SELECTION_DECREASE (2u)
#define MOTOR_SELECTION_STOP     (3u)

/*******************************************************************************
* Global variables
*******************************************************************************/
FTM_Type * const ftmBase = FTM0;
const IRQn_Type ftmIRQ = FTM0_Ovf_Reload_IRQn;
const IRQn_Type ftmIRQ_channel5 = FTM0_Ch4_Ch5_IRQn;
uint16_t timerCounterValue[2] = {0u};
uint16_t timerOverflowInterruptCount = 0u;
uint32_t count = 0u;
uint32_t count_ms = 0u;
uint8_t sendWakeupFlag = 0u;
l_u8 Motor1_Selection;
l_u8 Motor1_temp = 30u;

/*******************************************************************************
* Function prototypes
*******************************************************************************/
/* Callback function to get time interval in nano seconds */
uint32_t timerGetTimeIntervalCallback0(uint32_t *ns);
/* FTM initialization function */
void FTM_init(void);
/* Button IRQ Handler */
void BTNPORT_IRQHandler(void);

/*******************************************************************************
* Main
*******************************************************************************/
void lin_slave_start(void)
{
    /* Initialize and configure clocks
     *  -   Setup system clocks, dividers
     *  -   Configure LPUART clock, GPIO clock
     *  -   see clock manager component for more details
     */
    CLOCK_SYS_Init(g_clockManConfigsArr, CLOCK_MANAGER_CONFIG_CNT,
                        g_clockManCallbacksArr, CLOCK_MANAGER_CALLBACK_CNT);
    CLOCK_SYS_UpdateConfiguration(0U, CLOCK_MANAGER_POLICY_AGREEMENT);

    /* Initialize pins
     *  -   Init LPUART and GPIO pins
     *  -   See PinSettings component for more info
     */
    PINS_DRV_Init(NUM_OF_CONFIGURED_PINS, g_pin_mux_InitConfigArr);

    /* Configure pin directions for buttons and leds */
    GPIO_HAL_SetPinsDirection(LED_GPIO_PORT, PORT_LED0_MASK | PORT_LED1_MASK | PORT_LED2_MASK);
    GPIO_HAL_SetPinsDirection(BTN_GPIO_PORT, ~(PORT_BTN0_MASK | PORT_BTN1_MASK));

#if USE_LIN_XCVR
    /* Set LIN transceiver sleep pin direction */
    GPIO_HAL_SetPinsDirection(LIN_XCVR_ENABLE_GPIO_PORT, LIN_XCVR_ENABLE_MASK);
    /* Wake up LIN transceiver */
    GPIO_HAL_SetPins(LIN_XCVR_ENABLE_GPIO_PORT, LIN_XCVR_ENABLE_MASK);
#endif

    INT_SYS_InstallHandler(BTN_PORT_IRQn, BTNPORT_IRQHandler, (isr_t *)NULL);
    INT_SYS_EnableIRQ(BTN_PORT_IRQn);

    FTM_init();
    l_sys_init();
    l_ifc_init(LI0);

    /* Infinite loop */
    for (; ; )
    {
        /* Check if temp signal is updated */
        if (l_flg_tst_LI0_Motor1Selection_flag())
        {
            /* Clear this flag... */
            l_flg_clr_LI0_Motor1Selection_flag();

            /* Store selection data */
            Motor1_Selection = l_u8_rd_LI0_Motor1Selection();

            /* The application will change Motor selection in case
            the temperature is greater than maximum value to release motor power
            This will be transfered by unconditional frame type in LIN bus */
            l_u8_wr_LI0_Motor1Temp(Motor1_temp);

            /* Check if power off motor due to high temperature */
            if (Motor1_Selection == MOTOR_SELECTION_STOP)
            {
                RGB_RED_ON;
                RGB_BLUE_OFF;
                RGB_GREEN_OFF;
            }

            if (Motor1_Selection == MOTOR_SELECTION_INCREASE)
            {
                RGB_GREEN_ON;
                RGB_BLUE_OFF;
                RGB_RED_OFF;
            }

            if (Motor1_Selection == MOTOR_SELECTION_DECREASE)
            {
                RGB_BLUE_ON;
                RGB_GREEN_OFF;
                RGB_RED_OFF;
            }
        }

        /* Check node state */
        if (LIN_NODE_STATE_SLEEP_MODE == lin_lld_get_state(LI0))
        {
            RGB_RED_OFF;
            RGB_BLUE_OFF;
            RGB_GREEN_OFF;
        }
    }
}


/*******************************************************************************
* Code
*******************************************************************************/

/*******************************************************************************
* Initializes FTM timer
*******************************************************************************/
void FTM_init(void)
{
    /* Enable Flex Timer clock */
    PCC_HAL_SetClockMode(PCC, PCC_FLEXTMR0_CLOCK, true);
    /* Choose PTB17 to input capture pin and should be connected to RX pin of instance LPUART */
    PORT_HAL_SetMuxModeSel(PORTB, 17U, PORT_MUX_ALT2);
    /* Select Timer Prescale equal to 1 */
    FTM_HAL_SetClockPs(ftmBase, FTM_CLOCK_DIVID_BY_1);
    /* Set input capture mode */
    FTM_HAL_SetChnMSnBAMode(ftmBase, 5U, 0U);
    FTM_HAL_SetChnEdgeLevel(ftmBase, 5U, 3U);
    /* Interrupt every 500us */
    FTM_HAL_SetMod(ftmBase,TIMER_VALUE);
    FTM_HAL_SetCounterInitVal(ftmBase, 0U);
    /* Enable Flex Timer clock */
    FTM_HAL_SetClockSource(ftmBase, FTM_CLOCK_SOURCE_SYSTEMCLK);
    /* Enable channel 5 interrupt */
    FTM_HAL_EnableChnInt(ftmBase, 5U);
    /* Enable timer overflow interrupt */
    FTM_HAL_SetTimerOverflowInt(ftmBase, true);
    /* Enable interrupt NVIC */
    INT_SYS_EnableIRQ(ftmIRQ);
    INT_SYS_EnableIRQ(ftmIRQ_channel5);
}

/*******************************************************************************
* Callback function to get time interval in nano seconds
*******************************************************************************/
uint32_t timerGetTimeIntervalCallback0(uint32_t *ns)
{
    timerCounterValue[1] = FTM_HAL_GetChnCountVal(ftmBase,5U);
    *ns = ((uint32_t)(timerCounterValue[1] + timerOverflowInterruptCount*TIMER_VALUE - timerCounterValue[0]))*1000 / TIMER_1US;
    timerOverflowInterruptCount = 0u;
    timerCounterValue[0] = timerCounterValue[1];
    return 0u;
}

/*******************************************************************************
* FTM Handler
*******************************************************************************/
void FTM0_Ovf_Reload_IRQHandler(void)
{
    /* Timer overflow */
    if (FTM0->SC & FTM_SC_TOF_MASK)
    {
        FTM_HAL_ClearTimerOverflow(ftmBase);
        timerOverflowInterruptCount++;

        /* Timer Interrupt Handler */
        lin_lld_timeout_service(LI0);

        /* Toggle LED every 0.1s */
        if (++count >= 200u)
        {
            count = 0u;

            /* Increase temp */
            if (Motor1_Selection == MOTOR_SELECTION_INCREASE)
            {
                Motor1_temp++;
            }
            /* Decrease temp */
            else if (Motor1_Selection == MOTOR_SELECTION_DECREASE)
            {
                Motor1_temp--;
            }

        }
    }
}

void FTM0_Ch4_Ch5_IRQHandler(void)
{
	/* Clear interrupt flag */
	FTM_HAL_ClearChnEventFlag(ftmBase, 5U);

	if (LIN_DRV_AutoBaudCapture(INST_LIN1) == STATUS_SUCCESS)
	{
		/* Disable channel 5 interrupt */
	    FTM_HAL_DisableChnInt(ftmBase, 5U);
	    /* Disable interrupt NVIC */
	    INT_SYS_DisableIRQ(ftmIRQ_channel5);
	}
}

/*******************************************************************************
* PORT Handler
*******************************************************************************/
void BTNPORT_IRQHandler(void)
{
    /* If SW2/Button 1 is pressed */
    if (PORT_HAL_GetPinIntMode(BTN_PORT_NAME, PORT_BTN0_INDEX))
    {
        PORT_HAL_ClearPinIntFlagCmd(BTN_PORT_NAME, PORT_BTN0_INDEX);
        if (Motor1_temp < 255U)
        {
          Motor1_temp += 60U;
        }
    }

    /* If SW3/Button 2 is pressed */
    if (PORT_HAL_GetPinIntMode(BTN_PORT_NAME, PORT_BTN1_INDEX))
    {
        PORT_HAL_ClearPinIntFlagCmd(BTN_PORT_NAME, PORT_BTN1_INDEX);
        Motor1_temp = 30U;
    }
}

/*******************************************************************************
 * EOF
 ******************************************************************************/
