/*
 * Copyright (c) 2015 - 2016 , Freescale Semiconductor, Inc.
 * Copyright 2016-2017 NXP
 * All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NXP "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NXP OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

/* Including needed modules to compile this module/procedure */
#include "Cpu.h"
#include "clockMan1.h"
#include "pin_mux.h"
#include "osif1.h"
#include "sbc_uja11691.h"
#include "lpspiCom1.h"
#include "pwrMan1.h"
#include "lpit1.h"
#include "lpit1.h"
#include "dmaController1.h"

#if CPU_INIT_CONFIG
#include "Init_Config.h"
#endif

volatile int exit_code = 0U;
/* User includes (#include below this line is not maintained by Processor Expert) */

/* This example is setup to work by default with EVB. To use it with other boards
 please comment the following line
 */

#define DEV_ERROR_DETECT

#define RXD_GPIO_MASK      0x0010U
#define RXD_PORT           PORTE
#define RXD_GPIO           PTE
#define RXD                (4U)
#define SW3_GPIO           PTC
#define SW3_PORT           PORTC
#define SW3_GPIO_MASK      0x2000U
#define SW3               (13U)
#define SW2_GPIO          PTC
#define SW2_PORT          PORTC
#define SW2_GPIO_MASK     0x1000U
#define SW2               (12U)
#define LED_PORT          PORTD
#define LED_GPIO          PTD
#define PCC_CLOCK         PCC_PORTD_CLOCK
#define LED_RED           (15U)
#define LED_GREEN         (16U)
#define LED_BLUE          (0U)
#define LPIT_CHANNEL      (0U)
#define POWER_MODES_NUM   (2u)
#define RUN               (0u) /* High speed run      */
#define SLEEP             (1u) /* Very low power stop */

uint8_t timerCount = 0;
volatile uint8_t wakeUpEvnt = false;

/* @brief Interrupt service routine for RDX.
 * MCU is wake-up using CAN wake-up signal. */
static void CanWakeUpISR(void);

/* @brief Interrupt service routine for WatchDog refreshing. */
static void SBC_FeedWachdogAndLeds(void);

/* Interrupt service routine for RDX.
 * MCU is wake-up using CAN wake-up signal. */
static void CanWakeUpISR(void)
{
    /* Clear button IRQ flag */
    PORT_HAL_ClearPortIntFlagCmd(RXD_PORT);
    /* Flag for wake up event. */
    wakeUpEvnt = true;
    /* Start Feeding of WachDog by LPIT timer interrupts. */
    LPIT_DRV_StartTimerChannels(INST_LPIT1, 1U << LPIT_CHANNEL);    /* Check if some event on SBC occurred. */
}

/* Interrupt service routine for WachDog refreshing. */
static void SBC_FeedWachdogAndLeds(void)
{
    SBC_FeedWatchdog();
    /* Toggle with blue led to show that WachDog is triggered. */
    GPIO_HAL_TogglePins(LED_GPIO, 1 << LED_BLUE);

    /* Clear interrupt flag.*/
    LPIT_DRV_ClearInterruptFlagTimerChannels(INST_LPIT1, 1 << LPIT_CHANNEL);
}

/*!
 \brief The main function for the project.
 \details The startup initialization sequence is the following:
 * - __start (startup asm routine)
 * - __init_hardware()
 * - main()
 *   - PE_low_level_init()
 *     - Common_Init()
 *     - Peripherals_Init()
 */
int main(void)
{

    /*** Processor Expert internal initialization. DON'T REMOVE THIS CODE!!! ***/
#ifdef PEX_RTOS_INIT
    PEX_RTOS_INIT(); /* Initialization of the selected RTOS. Macro is defined by the RTOS component. */
#endif
    /*** End of Processor Expert internal initialization.                    ***/

    /* Write your code here */
    /* Initialize and configure clocks
     * 	-	see clock manager component for details
     */
    status_t status = STATUS_SUCCESS;
    sbc_factories_conf_t factoriesConf;

    /* Initialization clocks . */
    CLOCK_SYS_Init(g_clockManConfigsArr, CLOCK_MANAGER_CONFIG_CNT,
            g_clockManCallbacksArr, CLOCK_MANAGER_CALLBACK_CNT);
    CLOCK_SYS_UpdateConfiguration(0U, POWER_MANAGER_POLICY_AGREEMENT);

    /* Initialize pins
     *	-	See PinSettings component for more info
     */
    PINS_DRV_Init(NUM_OF_CONFIGURED_PINS, g_pin_mux_InitConfigArr);

    /* Initialize Power Manager */
    POWER_SYS_Init(&powerConfigsArr, POWER_MODES_NUM, &powerStaticCallbacksConfigsArr, 0);

    /* Install hander for CanWakeUp pin ISR. For wake up event is used RXD -
     * SBC_CAN_RX pin of SBC_UJA1669. This pin is used for wake up MCU
     * over CAN. There are only interrupt pin instead of Can driver in order
     * to fit simpler example and make impress on state machine.
     * This pin is assert low when wake up event occurred.
     * Wake up event can occurred if there some error condition or using
     * WAKE pin of SBC or over can Can.
     *  */
    INT_SYS_InstallHandler(PORTE_IRQn, CanWakeUpISR, (isr_t *)0);
    /* Enable button port IRQ */
    INT_SYS_EnableIRQ(PORTE_IRQn);

    INT_SYS_SetPriority(PORTE_IRQn, 2);

    /* Output direction for RGB LED. */
    GPIO_HAL_SetPinsDirection(LED_GPIO, ((1 << LED_RED) | (1 << LED_GREEN) | (1 << LED_BLUE)));
    /* Set input direction for SW2. */
    GPIO_HAL_SetPinsDirection(SW2_GPIO, (0 << SW2));
    /* Set input direction for SW2. */
    GPIO_HAL_SetPinsDirection(SW3_GPIO, (0 << SW3));
    /* Set input direction for SBC_CAN_RX. */
    GPIO_HAL_SetPinsDirection(RXD_GPIO, (0 << RXD));

    /* Set Output value LED0 & LED_RED */
    GPIO_HAL_SetPins(LED_GPIO, 1 << LED_RED);
    GPIO_HAL_SetPins(LED_GPIO, 1 << LED_GREEN);
    GPIO_HAL_SetPins(LED_GPIO, 1 << LED_BLUE);

    INT_SYS_SetPriority(LPSPI1_IRQn, 1);

    status = LPSPI_DRV_MasterInit(LPSPICOM1, &lpspiCom1State, &lpspiCom1_MasterConfig0);

    DEV_ASSERT(status == STATUS_SUCCESS);

    /* PIT initialization for periodic 100ms WachDog refresh. */

    /* Initialize LPIT instance 0
     *  -   Reset and enable peripheral
     */
    LPIT_DRV_Init(INST_LPIT1, &lpit1_InitConfig);
    /* Initialize LPIT channel 0 and configure it as a periodic counter
     * which is used to generate an interrupt every second.
     */

    /* Channel 1 configuration. */
    LPIT_DRV_InitChannel(INST_LPIT1, LPIT_CHANNEL, &lpit1_ChnConfig0);

    /* Install LPIT_ISR as LPIT interrupt handler */
    INT_SYS_InstallHandler(LPIT0_Ch0_IRQn, &SBC_FeedWachdogAndLeds, (isr_t *) 0);
    /* Enable LPIT interrupt. */
    INT_SYS_EnableIRQ(LPIT0_Ch0_IRQn);
    /* Set Priority lower to third place. */
    INT_SYS_SetPriority(LPIT0_Ch0_IRQn, 3U);

    /* Initialize SBC. */
    status = SBC_Init(&sbc_uja11691_InitConfig0, LPSPICOM1);

    /* Start Feeding of WachDog by LPIT timer interrupts. */
    LPIT_DRV_StartTimerChannels(INST_LPIT1, 1U << LPIT_CHANNEL);

    DEV_ASSERT(status == STATUS_SUCCESS);

    /* Test require correct factory settings for example.
     * FNMC bit must be disabled, SBC_UJA_SBC_SDMC_DIS must be disabled
     * and slpc must be allowed. */
    status = SBC_GetFactoriesSettings(&factoriesConf);
    DEV_ASSERT(status == STATUS_SUCCESS);
    if(factoriesConf.control.fnmc == SBC_UJA_SBC_FNMC_EN ||
            factoriesConf.control.sdmc == SBC_UJA_SBC_SDMC_EN ||
            factoriesConf.control.slpc == SBC_UJA_SBC_SLPC_IG)
    {
        /* Set correct settings. */
        factoriesConf.control.fnmc = SBC_UJA_SBC_FNMC_DIS;
        factoriesConf.control.sdmc = SBC_UJA_SBC_SDMC_DIS;
        factoriesConf.control.slpc = SBC_UJA_SBC_SLPC_AC;

        /* If this function return SBC_NVN_ERROR. Factory settings must be preset
         * to default state by Hardware configuration. Please read
         * SBC_ChangeFactoriesSettings descriptions for correct settings.
         * Conditions for Hardware reset value are
         * -pin RSTN is held LOW
         * -CANH is pulled up to VBAT
         * -CANL is pulled down to GND
         * When HW reset is done you can change factory settings by running
         * this example. Be carefully device will be reseted,
         * initialization sequence must be done again. Run the example again
         * with new configuration.
         * */
        status = SBC_ChangeFactoriesSettings(&factoriesConf);
        DEV_ASSERT(status == STATUS_SUCCESS);
    }

    /* Set interrupt priorities. */
    INT_SYS_SetPriority(SysTick_IRQn, 0);

    /* SBC has two power save mode. StandBy mode is first power safe mode.
     * MCU stay powered by SBC. In this example SBC is set to StandBy mode and
     * MCU moves to deeply sleep mode. MCU is wake up by SBC_CAN_RX(RXD) where Low
     * level is occurred if there are some wake up event on SBC otherwise it is High.
     * Use WAKE (SW9 button) pin or external CAN to wake up MCU.
     * Sleep mode is second mode which turn of power for MCU.
     * Use SW9 button for wake up SBC.
     * Move SBC to StandBy mode using SW2 button. Use SW9 button for wake up
     * event. Press SW3 button to move to Sleep mode and use SW9
     * for wake up device.
     * LED_GREEN is on in Normal mode otherwise is off.
     * LED_RED is on in StandBy mode otherwise is off.
     * LED_BLUE is toggle when MCU run and SBC_FeedWatchdog is called.
     * SBC WatchDog is in autonomous mode. The WatchDog is turn of
     * in Stand by mode if RXD is High
     *  */

    sbc_mode_mc_t mode = SBC_UJA_MODE_MC_SLEEP;
    sbc_evn_capt_t event;
    while(1)
    {
        /* Go to StandBy mode if SW2 is preset. */
        if((GPIO_HAL_ReadPins(SW2_GPIO)& SW2_GPIO_MASK) >> SW2)
        {
            status = SBC_SetMode(SBC_UJA_MODE_MC_STANDBY);

            DEV_ASSERT(status == STATUS_SUCCESS);
        }

        /* Go to normal mode from SBC wake up event. */
        if(wakeUpEvnt == true)
        {
            status = SBC_SetMode(SBC_UJA_MODE_MC_NORMAL);
            DEV_ASSERT(status == STATUS_SUCCESS);

            status = SBC_GetEventsStatus(&event);
            DEV_ASSERT(status == STATUS_SUCCESS);
            /* Clean Events if some occurred. */
            SBC_CleanEvents(&event);

            DEV_ASSERT(status == STATUS_SUCCESS);

            wakeUpEvnt = false;
        }

        /* Go to sleep mode if SW3 is preset. */
        if((GPIO_HAL_ReadPins(SW3_GPIO)& SW3_GPIO_MASK) >> SW3)
        {
            status = SBC_SetMode(SBC_UJA_MODE_MC_SLEEP);
            DEV_ASSERT(status == STATUS_SUCCESS);
        }

        /* Check SBC mode each 400ms. */
        if(timerCount >= 2)
        {
            status = SBC_GetMode(&mode);
            if(( mode== SBC_UJA_MODE_MC_NORMAL))
            {
                /* Turn on Green Led. */
                GPIO_HAL_ClearPins(LED_GPIO, 1 << LED_GREEN);
                /* Turn off Red Led. */
                GPIO_HAL_SetPins(LED_GPIO, 1 << LED_RED);
            }
            else
            {
                /* Turn off Green Led. */
                GPIO_HAL_ClearPins(LED_GPIO, 1 << LED_RED);
                /* Turn on Red Led. */
                GPIO_HAL_SetPins(LED_GPIO, 1 << LED_GREEN);
                /* Move MCU to sleep mode. */
               if((GPIO_HAL_ReadPins(RXD_GPIO)& RXD_GPIO_MASK) >> RXD)
               {
                   /* Turn off timer if RDX is HIGH in sleep mode. WachDog is off
                   * in autonomous mode when stand by mode is set and RXD is high. */
                   LPIT_DRV_StopTimerChannels(INST_LPIT1, 1U << LPIT_CHANNEL);
                   /* Wait second in the StandBy mode to demonstrate WachDog
                    * is not needed. */
                   OSIF_TimeDelay(1000);
               }
               /* Turn off the Red led for power saving. */
               GPIO_HAL_SetPins(LED_GPIO, 1 << LED_RED);
               /* Move Mcu to sleep mode. */
               status = POWER_SYS_SetMode(SLEEP, POWER_MANAGER_POLICY_AGREEMENT);
               DEV_ASSERT(status == STATUS_SUCCESS);
            }
            timerCount = 0;
        }

        DEV_ASSERT(status == STATUS_SUCCESS);
        timerCount++;
        /* Wait 200ms before new loop is started. */
        OSIF_TimeDelay(200);
    }

    /* Unused variable. It is used only for test reason. */
    (void )status;

    /*** Don't write any code pass this line, or it will be deleted during code generation. ***/
  /*** RTOS startup code. Macro PEX_RTOS_START is defined by the RTOS component. DON'T MODIFY THIS CODE!!! ***/
  #ifdef PEX_RTOS_START
    PEX_RTOS_START();                  /* Startup of the selected RTOS. Macro is defined by the RTOS component. */
  #endif
  /*** End of RTOS startup code.  ***/
  /*** Processor Expert end of main routine. DON'T MODIFY THIS CODE!!! ***/
  for(;;) {
    if(exit_code != 0) {
      break;
    }
  }
  return exit_code;
  /*** Processor Expert end of main routine. DON'T WRITE CODE BELOW!!! ***/
} /*** End of main routine. DO NOT MODIFY THIS TEXT!!! ***/

/* END main */
/*!
 ** @}
 */
/*
 ** ###################################################################
 **
 **     This file was created by Processor Expert 10.1 [05.21]
 **     for the Freescale S32K series of microcontrollers.
 **
 ** ###################################################################
 */
