/*
 * Copyright (c) 2015 - 2016 , Freescale Semiconductor, Inc.
 * Copyright 2016-2017 NXP
 * All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NXP "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NXP OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
/* ###################################################################
**     Filename    : main.c
**     Project     : ewm_interrupt
**     Processor   : S32K144_100
**     Version     : Driver 01.00
**     Compiler    : GNU C Compiler
**     Date/Time   : 2016-10-03, 10:53, # CodeGen: 0
**     Abstract    :
**         Main module.
**         This module contains user's application code.
**     Settings    :
**     Contents    :
**         No public methods
**
** ###################################################################*/
/*!
** @file main.c
** @version 01.00
** @brief
**         Main module.
**         This module contains user's application code.
*/
/*!
**  @addtogroup main_module main module documentation
**  @{
*/
/* MODULE main */


/* Including needed modules to compile this module/procedure */
#include "Cpu.h"
#include "clockMan1.h"
#include "pin_mux.h"
#include "extWdog1.h"
#if CPU_INIT_CONFIG
  #include "Init_Config.h"
#endif

volatile int exit_code = 0;
/* User includes (#include below this line is not maintained by Processor Expert) */

#include <stdint.h>
#include <stdbool.h>

/* To use with other board than EVB please comment the following line */
#define EVB

#ifdef EVB
    #define LED0            15U
    #define LED1            16U
    #define LED_GPIO        PTD

#else
    #define LED0             0U
    #define LED1             1U
    #define LED_GPIO        PTC
#endif

/* Initialize SysTick counter */
void SysTick_Init(void)
{
	S32_SysTick->RVR = 0xFFFFFFul;
	S32_SysTick->CVR = 0ul;
	S32_SysTick->CSR = 0u;
}

/* Enable SysTick counter and interrupt */
void SysTick_Enable(void)
{
	S32_SysTick->CSR = S32_SysTick_CSR_TICKINT(1u) | S32_SysTick_CSR_ENABLE(1);
}

/* Disable SysTick */
void SysTick_Disable(void)
{
	S32_SysTick->CSR = 0ul;
}

/* SysTick IRQ handler */
void SysTick_Handler(void)
{
	/* Variable that stores the number of EWM refreshes */
	static uint8_t s_refreshNumber = 0;
	/* After 30 watchdog refreshes, the interrupt will no longer reset the wdog counter */
	if(s_refreshNumber < 30)
	{
		/* Reset Watchdog counter */
		EWM_DRV_Refresh(EXTWDOG1);
		/* Toggle LED0 */
		GPIO_HAL_TogglePins(LED_GPIO, (1 << LED0));
		/* Increment refresh number */
		s_refreshNumber++;
	}
}
/* EWM IRQ handler */
void EWM_ISR(void)
{
	/* Turn off both LEDs */
	GPIO_HAL_ClearPins(LED_GPIO, (1 << LED0) | (1 << LED1));
	/* Disable the SysTick timer */
	SysTick_Disable();
}

/*!
  \brief The main function for the project.
  \details The startup initialization sequence is the following:
 * - __start (startup asm routine)
 * - __init_hardware()
 * - main()
 *   - PE_low_level_init()
 *     - Common_Init()
 *     - Peripherals_Init()
*/
int main(void)
{
  /* Write your local variable definition here */

  /*** Processor Expert internal initialization. DON'T REMOVE THIS CODE!!! ***/
  #ifdef PEX_RTOS_INIT
    PEX_RTOS_INIT();                 /* Initialization of the selected RTOS. Macro is defined by the RTOS component. */
  #endif
  /*** End of Processor Expert internal initialization.                    ***/

    /* Initialize and configure clocks
     *  -   Setup system clocks
     *  -   Enable clock feed for Ports and EWM
     *  -   See Clock Manager component for more info
     */
    CLOCK_SYS_Init(g_clockManConfigsArr, CLOCK_MANAGER_CONFIG_CNT,
                   g_clockManCallbacksArr, CLOCK_MANAGER_CALLBACK_CNT);
    CLOCK_SYS_UpdateConfiguration(0U, CLOCK_MANAGER_POLICY_AGREEMENT);


    /* Initialize pins
     *  -   Setup pins for EWM
     *  -   Setup output pins for LEDs
     *  -   See PinSettings component for more info
     */
    PINS_DRV_Init(NUM_OF_CONFIGURED_PINS, g_pin_mux_InitConfigArr);

    /* Initialize GPIO for LEDs */
    GPIO_HAL_SetPinsDirection(LED_GPIO, (1 << LED0) | (1 << LED1));
    /* Turn both LEDs off */
    GPIO_HAL_WritePin(LED_GPIO, LED0, 1u);
    GPIO_HAL_WritePin(LED_GPIO, LED1, 1u);

    /* Install IRQ Handlers for EWM and SysTick */
    INT_SYS_InstallHandler(WDOG_EWM_IRQn, EWM_ISR, (isr_t *)0);
    INT_SYS_InstallHandler(SysTick_IRQn, SysTick_Handler, (isr_t *)0);

    /* Enable EWM IRQ */
    INT_SYS_EnableIRQ(WDOG_EWM_IRQn);

    /* Initialize and enable SysTick */
    SysTick_Init();
    SysTick_Enable();

    /* Initialize EWM */
    EWM_DRV_Init(EXTWDOG1, &extWdog1_Config0);

    while(1)
    {
    	;
    }

  /*** Don't write any code pass this line, or it will be deleted during code generation. ***/
  /*** RTOS startup code. Macro PEX_RTOS_START is defined by the RTOS component. DON'T MODIFY THIS CODE!!! ***/
  #ifdef PEX_RTOS_START
    PEX_RTOS_START();                  /* Startup of the selected RTOS. Macro is defined by the RTOS component. */
  #endif
  /*** End of RTOS startup code.  ***/
  /*** Processor Expert end of main routine. DON'T MODIFY THIS CODE!!! ***/
  for(;;) {
    if(exit_code != 0) {
      break;
    }
  }
  return exit_code;
  /*** Processor Expert end of main routine. DON'T WRITE CODE BELOW!!! ***/
} /*** End of main routine. DO NOT MODIFY THIS TEXT!!! ***/

/* END main */
/*!
** @}
*/
/*
** ###################################################################
**
**     This file was created by Processor Expert 10.1 [05.21]
**     for the Freescale S32K series of microcontrollers.
**
** ###################################################################
*/

