/*
 * Copyright (c) 2015 - 2016 , Freescale Semiconductor, Inc.
 * Copyright 2016-2017 NXP
 * All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NXP "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NXP OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "board.h"
#include "memProtect1.h"
#include <stdint.h>
#include <stdbool.h>

/*******************************************************************************
 * Variables
 ******************************************************************************/
volatile uint32_t flagInitMPU = 0U;
mpu_access_err_info_t errorDetail;

/*******************************************************************************
 * Code
 ******************************************************************************/
/* Initializes clocks for MPU instances used */
void initClocks()
{
    /* Enable clock for MPU. */
    CLOCK_SYS_Init(g_clockManConfigsArr, CLOCK_MANAGER_CONFIG_CNT,
                        g_clockManCallbacksArr, CLOCK_MANAGER_CALLBACK_CNT);
    CLOCK_SYS_UpdateConfiguration(0U, CLOCK_MANAGER_POLICY_AGREEMENT);
}

/* Initializes LEDs */
void initLED()
{
    /* Setting clock gate for LEDs */
    PCC_HAL_SetClockMode(PCC, PCC_PORTD_CLOCK, true);

    /* Mux to gpio for LEDs */
    PORT_HAL_SetMuxModeSel(LED_RED_PORT, LED_RED_GPIO_PIN, PORT_MUX_AS_GPIO);
    PORT_HAL_SetDriveStrengthMode(LED_RED_PORT, LED_RED_GPIO_PIN, PORT_HIGH_DRIVE_STRENGTH);

    PORT_HAL_SetMuxModeSel(LED_GREEN_PORT, LED_GREEN_GPIO_PIN, PORT_MUX_AS_GPIO);
    PORT_HAL_SetDriveStrengthMode(LED_GREEN_PORT, LED_GREEN_GPIO_PIN, PORT_HIGH_DRIVE_STRENGTH);

    PORT_HAL_SetMuxModeSel(LED_BLUE_PORT, LED_BLUE_GPIO_PIN, PORT_MUX_AS_GPIO);
    PORT_HAL_SetDriveStrengthMode(LED_BLUE_PORT, LED_BLUE_GPIO_PIN, PORT_HIGH_DRIVE_STRENGTH);

    /* Output direction for LEDs */
    PTD->PDDR |= LED_RED_MASK | LED_GREEN_MASK | LED_BLUE_MASK;

    /* Initializes values */
    LED_RED_OFF;
    LED_GREEN_ON;
    LED_BLUE_OFF;
}

/* Initializes BUTTONs */
void initButton()
{
    /* Setting clock gate for buttons  */
    PCC_HAL_SetClockMode(PCC, PCC_PORTC_CLOCK, true);

    /* Mux to gpio for Buttons */
    PORT_HAL_SetMuxModeSel(SW2_PORT, SW2_GPIO_PIN, PORT_MUX_AS_GPIO);
    PORT_HAL_SetPullSel(SW2_PORT, SW2_GPIO_PIN, PORT_INTERNAL_PULL_UP_ENABLED);
    PORT_HAL_SetPinIntSel(SW2_PORT, SW2_GPIO_PIN, PORT_INT_FALLING_EDGE);

    PORT_HAL_SetMuxModeSel(SW3_PORT, SW3_GPIO_PIN, PORT_MUX_AS_GPIO);
    PORT_HAL_SetPullSel(SW3_PORT, SW3_GPIO_PIN, PORT_INTERNAL_PULL_UP_ENABLED);
    PORT_HAL_SetPinIntSel(SW3_PORT, SW3_GPIO_PIN, PORT_INT_FALLING_EDGE);

    /* Enable interrupt NVIC */
    INT_SYS_EnableIRQ(SW2_IRQ);
}

/******************************************************************************
 * PORT Handler
 ******************************************************************************/
void SW2_IRQ_HANDLER(void)
{
    /* If SW2/Button 0 is pressed */
    if (PORT_HAL_GetPinIntMode(SW2_PORT, SW2_GPIO_PIN))
    {
        PORT_HAL_ClearPinIntFlagCmd(SW2_PORT, SW2_GPIO_PIN);
        flagInitMPU = 1U;
    }

    /* If SW3/Button 1 is pressed */
    if (PORT_HAL_GetPinIntMode(SW3_PORT, SW3_GPIO_PIN))
    {
        PORT_HAL_ClearPinIntFlagCmd(SW3_PORT, SW3_GPIO_PIN);
    }
}

/*******************************************************************************
 * HardFault Handler
 ******************************************************************************/
void HardFault_Handler(void)
{
    /* Led status */
    LED_RED_ON;
    LED_GREEN_OFF;

    /* Get detail error access information */
    MPU_DRV_GetDetailErrorAccessInfo(MEMPROTECT1, FEATURE_MPU_SLAVE_FLASH_BOOTROM, &errorDetail);
}

/*******************************************************************************
 * EOF
 ******************************************************************************/
