/*
 * Copyright (c) 2015 - 2016 , Freescale Semiconductor, Inc.
 * Copyright 2016-2017 NXP
 * All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NXP "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NXP OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
/*!
 * @file board.h
 * @version 01.00
 * @brief Board configuration for CRC tests
 */

#ifndef BOARD_H
#define BOARD_H

#include "clock_manager.h"
#include "pin_mux.h"
#include "interrupt_manager.h"
#include "clockMan1.h"


/*******************************************************************************
 * Definitions
 ******************************************************************************/
/* LEDs */
#define LED_ON                  0U
#define LED_OFF                 1U
#define LED_RED_GPIO            PTD
#define LED_RED_PORT            PORTD
#define LED_RED_GPIO_PIN        15U
#define LED_RED_MASK            1U << LED_RED_GPIO_PIN
#define LED_GREEN_GPIO          PTD
#define LED_GREEN_PORT          PORTD
#define LED_GREEN_GPIO_PIN      16U
#define LED_GREEN_MASK          1U << LED_GREEN_GPIO_PIN
#define LED_BLUE_GPIO           PTD
#define LED_BLUE_PORT           PORTD
#define LED_BLUE_GPIO_PIN       0U
#define LED_BLUE_MASK           1U << LED_BLUE_GPIO_PIN

#define LED_RED_OFF         LED_RED_GPIO->PSOR |= LED_RED_MASK
#define LED_RED_ON          LED_RED_GPIO->PCOR |= LED_RED_MASK
#define LED_RED_TOGGLE      LED_RED_GPIO->PTOR |= LED_RED_MASK

#define LED_GREEN_OFF       LED_GREEN_GPIO->PSOR |= LED_GREEN_MASK
#define LED_GREEN_ON        LED_GREEN_GPIO->PCOR |= LED_GREEN_MASK
#define LED_GREEN_TOGGLE    LED_GREEN_GPIO->PTOR |= LED_GREEN_MASK

#define LED_BLUE_OFF        LED_BLUE_GPIO->PSOR |= LED_BLUE_MASK
#define LED_BLUE_ON         LED_BLUE_GPIO->PCOR |= LED_BLUE_MASK
#define LED_BLUE_TOGGLE     LED_BLUE_GPIO->PTOR |= LED_BLUE_MASK

/* SW */
#define SW2_GPIO            PTC
#define SW2_PORT            PORTC
#define SW2_GPIO_PIN        12U
#define SW2_IRQ             PORTC_IRQn
#define SW2_IRQ_HANDLER     PORTC_IRQHandler
#define SW2_IRQ_NAME        "SW2"

#define SW3_GPIO            PTC
#define SW3_PORT            PORTC
#define SW3_GPIO_PIN        13U
#define SW3_IRQ             PORTC_IRQn
#define SW3_IRQ_HANDLER     PORTC_IRQHandler
#define SW3_IRQ_NAME        "SW3"

/*******************************************************************************
 * API
 ******************************************************************************/
/* Initializes clocks for CRC instances used */
void initClocks(void);

/* Initializes LEDs */
void initLED(void);

/* Initializes BUTTONs */
void initButton(void);

#endif /* BOARD_H */
/*******************************************************************************
 * EOF
 ******************************************************************************/
