%-
%- Implementation of default RTOS ADAPTOR for bare-board configuration
%- (Special type of RTOS adapter used when no RTOS is active)
%-
%- (C) 2011 Freescale, all rights reserved
%-
%------------------------------------------------------------------------------
%- Including common RTOS adapter library.
%-----------------------------------------------------------------------------
%- It is included indirectly through symbol as a workaround of limitation
%- of the tool used for creating installation.
%-
%define RTOSAdap_priv_CommonAdapterInterface sw\RTOSAdaptor\Common_RTOSAdaptor.prg
%include %'RTOSAdap_priv_CommonAdapterInterface'
%undef RTOSAdap_priv_CommonAdapterInterface
%-
%-
%------------------------------------------------------------------------------
%SUBROUTINE RTOSAdap_genRTOSTypeDefinitions
%------------------------------------------------------------------------------
  %- PUBLIC TYPES
%{ {Default RTOS Adapter} RTOS specific definition of type of Ioctl() command constants %}
%SUBROUTINE_END
%-
%-
%------------------------------------------------------------------------------
%SUBROUTINE RTOSAdap_genDriverInstallationsFunctionDefinition
%------------------------------------------------------------------------------
%- For bare-board RTOS no driver installations are generated.
%-
%{ {Default RTOS Adapter} Function PE_LDD_driver_installations() definition: It is not generated %}
%SUBROUTINE_END
%-
%-
%------------------------------------------------------------------------------
%SUBROUTINE RTOSAdap_genDriverInstallationsFunctionDeclaration
%------------------------------------------------------------------------------
%- For bare-board RTOS no driver installations are generated.
%-
%{ {Default RTOS Adapter} Function PE_LDD_driver_installations() declaration: It is not generated %}
%SUBROUTINE_END
%-
%-
%------------------------------------------------------------------------------
%SUBROUTINE RTOSAdap_genDriverMemoryAlloc(arg_destPtrBuffer,arg_objType,opt_arg_errCode,arg_globDefsThread,opt_arg_MemoryAllocParams)
%------------------------------------------------------------------------------
  %-
  %inclSUB RTOSAdap_priv_getOptionalParameterEffectiveValue(componentInstanceName,,loc_componentInstanceName)
  %-
  %- Check if RTOSAdap_driverMemoryAlloc() was called only once for this object
  %if defined(RTOSAdap_alloc_object_%'arg_destPtrBuffer')
    %error! There was called allocation for object "%'arg_destPtrBuffer'" for object type "%'arg_objType'" twice (which does not work when the dynamic allocation is simulated only)
  %else
    %- o.k., it is first call, mark that such object was allocated
    %define RTOSAdap_alloc_object_%'arg_destPtrBuffer'
  %endif
  %-
  %THREAD %'arg_globDefsThread' SELECT
%>1%{ {Default RTOS Adapter} Static object used for simulation of dynamic driver memory allocation %}
  %THREAD %'arg_globDefsThread' UNSELECT
  %-
  %define! loc_ObjDefPrefix
  %define! loc_ObjDefSuffix
  %-
  %------------------------------------------------------------------------------
  %- Processing of optional parameters
  %------------------------------------------------------------------------------
  %inclSUB RTOSAdap_lib_getStructMember(%'opt_arg_MemoryAllocParams',ALIGN,loc_Param_Align,optional)
  %inclSUB RTOSAdap_lib_getStructMember(%'opt_arg_MemoryAllocParams',ZERO,loc_Param_Zero,optional)
  %------------------------------------------------------------------------------
  %- Processing of ALIGN optional parameter
  %------------------------------------------------------------------------------
  %if loc_Param_Align != 'undef'
    %if loc_Param_Align != ''
      %- Add new data section to linker file
      %if (Compiler = 'CodeWarriorARM') | (Compiler = 'CodeWarriorMCF')
        %add PE_G_LCF_DATA_SECTION .%'loc_componentInstanceName'_memory_section
        %add PE_G_LCF_DATA_ALIGN %loc_Param_Align
        %define! loc_ObjDefPrefix __declspec(%'loc_componentInstanceName'_memory_section)%_space_
      %elif (Compiler = 'IARARM')
        %add PE_G_LCF_DATA_SECTION %'loc_componentInstanceName'_memory_section
        %add PE_G_LCF_DATA_ALIGN %loc_Param_Align
      %elif (Compiler = 'IARMCF')
        %- No linker file modifications needed.
      %elif (Compiler = 'GNUC')
        %- No linker file modifications needed.
      %elif (Compiler = 'ARM_CC')
        %- No linker file modifications needed.
      %else
        %error! RTOS adapter: Unsupported compiler to allign allocated memory!
      %endif
      %THREAD %'arg_globDefsThread' SELECT
      %if (Compiler = 'CodeWarriorARM') | (Compiler = 'CodeWarriorMCF')
%>1%{ This pragma aligns an object to %loc_Param_Align bytes boundary. %}
%>1#pragma define_section %'loc_componentInstanceName'_memory_section ".%'loc_componentInstanceName'_memory_section" far_abs RW
      %elif (Compiler = 'IARARM')
%>1%{ This pragma aligns an object to %loc_Param_Align bytes boundary. %}
%>1#pragma location=".%'loc_componentInstanceName'_memory_section"
      %elif (Compiler = 'IARMCF')
%>1%{ This pragma aligns an object to %loc_Param_Align bytes boundary. %}
%>1#pragma data_alignment=%loc_Param_Align
      %elif (Compiler = 'GNUC')
        %define! loc_ObjDefSuffix %'_space_'__attribute__ ((aligned (%'loc_Param_Align')))
      %elif (Compiler = 'ARM_CC')
        %define! loc_ObjDefSuffix %'_space_'__attribute__ ((aligned (%'loc_Param_Align')))
      %endif
      %THREAD %'arg_globDefsThread' UNSELECT
    %else
      %error! Parameter ALIGN has no value
    %endif
    %undef loc_Param_Align
  %endif
  %-
  %- Generate defintion of static object
  %define loc_staticObjName %'arg_destPtrBuffer'__DEFAULT_RTOS_ALLOC
  %THREAD %'arg_globDefsThread' SELECT
%>1%'loc_ObjDefPrefix'static %'arg_objType' %'loc_staticObjName'%'loc_ObjDefSuffix';
  %THREAD %'arg_globDefsThread' UNSELECT
  %-
%{ {Default RTOS Adapter} Driver memory allocation: Dynamic allocation is simulated by a pointer to the static object %}
%'arg_destPtrBuffer' = &%'loc_staticObjName';
  %------------------------------------------------------------------------------
  %- Processing of ZERO optional parameter
  %------------------------------------------------------------------------------
  %if loc_Param_Zero != 'undef'
%{ {Default RTOS Adapter} Driver memory allocation: Fill the allocated memory by zero value %}
PE_FillMemory(%'arg_destPtrBuffer', 0U, sizeof(%'arg_objType'));
    %undef loc_Param_Zero
  %endif
  %-
  %undef loc_staticObjName
  %undef loc_ObjDefPrefix
%SUBROUTINE_END
%-
%-
%------------------------------------------------------------------------------
%SUBROUTINE RTOSAdap_genDriverMemoryDealloc(arg_ptrBuffer,arg_objType)
%------------------------------------------------------------------------------
  %- Bare-board RTOS is simulating dynamic allocation by static allocation.
  %- Allocation is only simulated, so deallocation has no effect.
%{ {Default RTOS Adapter} Driver memory deallocation: Dynamic allocation is simulated, no deallocation code is generated %}
%SUBROUTINE_END
%-
%-
%------------------------------------------------------------------------------
%SUBROUTINE RTOSAdap_getInterruptVectorSymbol(arg_vectorName,out_vectorIndexConstantName)
%------------------------------------------------------------------------------
  %- In default RTOS interrupt vector table is generated by PE and linked statically.
  %-
  %- Such symbol is not used in bare-board, but for syntax compatibility the
  %- sub-routine must return "something" valid.
  %-
  %define! %out_vectorIndexConstantName LDD_ivIndex_%'arg_vectorName'
%SUBROUTINE_END
%-
%-
%------------------------------------------------------------------------------
%SUBROUTINE RTOSAdap_genSetInterruptVector(arg_intVectorProperty,arg_isrFunctionName,arg_isrParameterType,arg_isrParameterValue,opt_arg_oldISRSettings,arg_globDefsThread)
%------------------------------------------------------------------------------
%- Bareboard arg_isrParameterType limitation: No. Parameter can be of any type.
%-
  %- In bareboard RTOS interrupt vector table is generated by PE and linked statically.
  %-
  %- Check if RTOSAdap_genSetInterruptVector() was called only once for this interrupt vector
  %if defined(RTOSAdap_alloc_interrupt_%"%'arg_intVectorProperty'_Name")
    %error! There was called allocation for interrupt vector %"%'arg_intVectorProperty'_Name" twice (which does not work for bareboard RTOS what only simulates the run-time vector allocation)
  %else
    %- o.k., it is first call, mark that interrupt vector was allocated
    %define RTOSAdap_alloc_interrupt_%"%'arg_intVectorProperty'_Name"
  %endif
  %-
  %- No ISR run-time allocation is used.
  %- Register ISR function to be statically generated into the vector table
  %if (%"%'arg_intVectorProperty'Shared" = 'no')
    %-Not shared interrupt vector
    %define_prj iv%"%'arg_intVectorProperty'_Name" %'arg_isrFunctionName'
  %else
    %- Shared interrupt vector
  %endif
  %-
  %if (arg_isrParameterType != '')
%{ {Default RTOS Adapter} Set interrupt vector: IVT is static, ISR parameter is passed by the global variable %}
%"%'arg_intVectorProperty'_Name"__DEFAULT_RTOS_ISRPARAM = %'arg_isrParameterValue';
  %-
  %- Generate the definition of the global variable holding ISR parameter value
    %THREAD %'arg_globDefsThread' SELECT
%>1%{ {Default RTOS Adapter} Global variable used for passing a parameter into ISR %}
%>1static %'arg_isrParameterType' %"%'arg_intVectorProperty'_Name"__DEFAULT_RTOS_ISRPARAM;
    %THREAD %'arg_globDefsThread' UNSELECT
  %endif
  %-
%SUBROUTINE_END
%-
%-
%------------------------------------------------------------------------------
%SUBROUTINE RTOSAdap_genRestoreInterruptVector(arg_intVectorProperty,arg_oldISRSettings)
%------------------------------------------------------------------------------
  %- In bareboard RTOS interrupt vector table is generated by PE and linked statically.
  %- No ISR run-time allocation is used, so deallocation has no effect.
%{ {Default RTOS Adapter} Restore interrupt vector: IVT is static, no code is generated %}
%SUBROUTINE_END
%-
%-
%------------------------------------------------------------------------------
%SUBROUTINE RTOSAdap_genISRSettingsVarDeclaration(arg_varName,opt_arg_comment)
%------------------------------------------------------------------------------
  %- In bareboard RTOS interrupt vector table is generated by PE and linked statically.
  %- No object for saving/restoring ISR vector is used, so no delcaration is generated
%SUBROUTINE_END
%-
%-
%------------------------------------------------------------------------------
%SUBROUTINE RTOSAdap_genISRFunctionDefinitionOpen(arg_intVectorProperty,arg_isrFunctionName,arg_isrParameterType,arg_isrParameterName)
%------------------------------------------------------------------------------
%- Bareboard arg_isrParameterType limitation: No. Parameter can be of any type.
%-
%if ((%'arg_intVectorProperty' = '$SHARED') | (%"%'arg_intVectorProperty'Shared" = 'no'))
  %- Not shared interrupt vector or shared interrupt handler defined in CPU component
%include Common\DefineISR.prg(%'arg_isrFunctionName',,%'arg_intVectorProperty')
%else
%include Common\DefineISR.prg(%'arg_isrFunctionName',,%'arg_intVectorProperty')
%endif
{
  %if arg_isrParameterName != ''
  %{ {Default RTOS Adapter} ISR parameter is passed through the global variable %}
  %'arg_isrParameterType' %'arg_isrParameterName' = %"%'arg_intVectorProperty'_Name"__DEFAULT_RTOS_ISRPARAM;
  %endif
%SUBROUTINE_END
%-
%-
%------------------------------------------------------------------------------
%SUBROUTINE RTOSAdap_genISRFunctionDefinitionClose(arg_intVectorProperty,arg_isrFunctionName,arg_isrParameterType,arg_isrParameterName)
%------------------------------------------------------------------------------
}
%SUBROUTINE_END
%-
%-
%------------------------------------------------------------------------------
%SUBROUTINE RTOSAdap_genISRFunctionDeclaration(arg_intVectorProperty,arg_isrFunctionName,arg_isrParameterType,arg_isrParameterName)
%------------------------------------------------------------------------------
%- Bareboard arg_isrParameterType limitation: No. Parameter can be of any type.
%-
%{ {Default RTOS Adapter} ISR function prototype %}
%if((%'arg_intVectorProperty' != '$SHARED') && (%'arg_intVectorProperty' != '') && (%'arg_isrFunctionName' != ''))
  %define! RTOSAdap_intVectorPropertySymbol_%"%'arg_intVectorProperty'_Name" %arg_intVectorProperty
  %define! RTOSAdap_intVectorName_%'arg_isrFunctionName' %"%'arg_intVectorProperty'_Name"
  %define! RTOSAdap_isrFunctionName_%"%'arg_intVectorProperty'_Name" %arg_isrFunctionName
%endif
%if ((%'arg_intVectorProperty' = '$SHARED') | (%"%'arg_intVectorProperty'Shared" = 'no'))
  %- Not shared interrupt vector
%include Common\CreateIntSection.prg
%include Common\DeclareISR.prg(%'arg_isrFunctionName',,%'arg_intVectorProperty')
%include Common\CreateCodeSection.prg
%else
  %- Shared interrupt vector
%include Common\DeclareISR.prg(%'arg_isrFunctionName',,%'arg_intVectorProperty')
%endif
%SUBROUTINE_END
%-
%-
%------------------------------------------------------------------------------
%SUBROUTINE RTOSAdap_genReentrantMethodBegin(opt_arg_genReentrantMethods)
%------------------------------------------------------------------------------
  %- Handle optional parameter
  %inclSUB RTOSAdap_priv_getOptionalParameterEffectiveValue(genReentrantMethods,%'opt_arg_genReentrantMethods',arg_genReentrantMethods)
  %- In bareboard use standard Processor Expert function generated into PE_Types.h.
  %if arg_genReentrantMethods == 'yes'
    %- Reentrant methods are required, generate enter critical section
%{ {Default RTOS Adapter} Method is required to be reentrant, critical section begins, general PE function is used %}
EnterCritical();
  %elif arg_genReentrantMethods == 'no'
    %- Reentrant methods are not required, no code is generated
  %else
    %error! Invalid value of parameter 'opt_arg_genReentrantMethods' ("%'arg_genReentrantMethods'")
  %endif
  %-
  %undef arg_genReentrantMethods
%SUBROUTINE_END
%-
%-
%------------------------------------------------------------------------------
%SUBROUTINE RTOSAdap_genReentrantMethodEnd(opt_arg_genReentrantMethods)
%------------------------------------------------------------------------------
  %- Handle optional parameter
  %inclSUB RTOSAdap_priv_getOptionalParameterEffectiveValue(genReentrantMethods,%'opt_arg_genReentrantMethods',arg_genReentrantMethods)
  %- In bareboard use standard Processor Expert function generated into PE_Types.h.
  %if arg_genReentrantMethods == 'yes'
    %- Reentrant methods are required, generate exit critical section
%{ {Default RTOS Adapter} Critical section end, general PE function is used %}
ExitCritical();
  %elif arg_genReentrantMethods == 'no'
    %- Reentrant methods are not required, no code is generated
  %else
    %error! Invalid value of parameter 'opt_arg_genReentrantMethods' ("%'arg_genReentrantMethods'")
  %endif
  %-
  %undef arg_genReentrantMethods
%SUBROUTINE_END
%-
%-
%------------------------------------------------------------------------------
%SUBROUTINE RTOSAdap_genCriticalSectionBegin(opt_arg_genReentrantMethods)
%------------------------------------------------------------------------------
  %- Handle optional parameter
  %inclSUB RTOSAdap_priv_getOptionalParameterEffectiveValue(genReentrantMethods,%'opt_arg_genReentrantMethods',arg_genReentrantMethods)
  %- In bareboard use standard Processor Expert function generated into PE_Types.h.
  %if arg_genReentrantMethods == 'yes'
    %- The method itself is generated as reentrant, additional critical sections are not needed
  %elif arg_genReentrantMethods == 'no'
    %- Reentrant methods are not required, but critical sections must remain
%{ {Default RTOS Adapter} Critical section begin, general PE function is used %}
EnterCritical();
  %else
    %error! Invalid value of parameter 'opt_arg_genReentrantMethods' ("%'arg_genReentrantMethods'")
  %endif
  %-
  %undef arg_genReentrantMethods
%SUBROUTINE_END
%-
%-
%------------------------------------------------------------------------------
%SUBROUTINE RTOSAdap_genCriticalSectionEnd(opt_arg_genReentrantMethods)
%------------------------------------------------------------------------------
  %- Handle optional parameter
  %inclSUB RTOSAdap_priv_getOptionalParameterEffectiveValue(genReentrantMethods,%'opt_arg_genReentrantMethods',arg_genReentrantMethods)
  %- In bareboard use standard Processor Expert function generated into PE_Types.h.
  %if arg_genReentrantMethods == 'yes'
    %- The method itself is generated as reentrant, additional critical sections are not needed
  %elif arg_genReentrantMethods == 'no'
    %- Reentrant methods are not required, but critical sections must remain
%{ {Default RTOS Adapter} Critical section end, general PE function is used %}
ExitCritical();
  %else
    %error! Invalid value of parameter 'opt_arg_genReentrantMethods' ("%'arg_genReentrantMethods'")
  %endif
  %-
  %undef arg_genReentrantMethods
%SUBROUTINE_END
%-
%-
%------------------------------------------------------------------------------
%SUBROUTINE RTOSAdap_genIoctlCommandConstantDefinition(arg_commandName,arg_commandLogicalIndex,opt_arg_simpleComponentType)
%------------------------------------------------------------------------------
  %- Get constant name
  %inclSUB RTOSAdap_getIoctlCommandConstantName(%'arg_commandName',%'opt_arg_simpleComponentType',loc_constantName)
  %-
#define %'loc_constantName'                                      %>57((LDD_RTOS_TIoctlCommand)%'arg_commandLogicalIndex')
  %-
  %undef loc_constantName
%SUBROUTINE_END
%-
%-
%------------------------------------------------------------------------------
%SUBROUTINE RTOSAdap_getIoctlCommandConstantName(arg_commandName,opt_arg_simpleComponentType,out_constantName)
%------------------------------------------------------------------------------
  %- Handle optional parameters
  %inclSUB RTOSAdap_priv_getOptionalParameterEffectiveValue(simpleComponentType,%'opt_arg_simpleComponentType',arg_simpleComponentType)
  %-
  %- Check if arg_componentType is a valid enumerated value
  %:loc_simpleCompTypeIndex = %get_index1(%'arg_simpleComponentType',RTOSAdap_enum_simpleComponentTypes)
  %if loc_simpleCompTypeIndex = '-1'
    %error! Invalid HAL simple component type "%'arg_simpleComponentType'"
  %endif
  %-
  %define! %'out_constantName' %'arg_simpleComponentType'_IOCTL_%'arg_commandName'
  %-
  %undef arg_simpleComponentType
  %undef loc_simpleCompTypeIndex
%SUBROUTINE_END
%-
%-
%------------------------------------------------------------------------------
%SUBROUTINE RTOSAdap_genRTOSIncludes()
%------------------------------------------------------------------------------
  %- In bareboard no RTOS includes are needed
%{ {Default RTOS Adapter} No RTOS includes %}
%SUBROUTINE_END
%-
%-
%------------------------------------------------------------------------------
%SUBROUTINE RTOSAdap_genRTOSDriverIncludes()
%------------------------------------------------------------------------------
  %- In bareboard no RTOS driver's includes are needed
%{ {Default RTOS Adapter} No RTOS driver includes %}
%SUBROUTINE_END
%-
%-
%------------------------------------------------------------------------------
%SUBROUTINE RTOSAdap_genPublishedRTOSSettings
%------------------------------------------------------------------------------
  %- In bareboard there are no published RTOS properties.
%{ {Default RTOS Adapter} No published RTOS settings %}
%SUBROUTINE_END
%-
%-
%------------------------------------------------------------------------------
%SUBROUTINE RTOSAdap_genMemoryBarrierBegin(opt_arg_genReentrantMethods)
%------------------------------------------------------------------------------
%SUBROUTINE_END
%-
%-
%------------------------------------------------------------------------------
%SUBROUTINE RTOSAdap_genMemoryBarrierEnd(opt_arg_genReentrantMethods)
%------------------------------------------------------------------------------
%SUBROUTINE_END
%-
%-
%------------------------------------------------------------------------------
%SUBROUTINE RTOSAdap_genReentrantMethodDeclare(opt_arg_genReentrantMethods, spinlock_name)
%------------------------------------------------------------------------------
 %- Do nothing for bareboard
%SUBROUTINE_END
%-
%-
%------------------------------------------------------------------------------
%SUBROUTINE RTOSAdap_genRTOSReturnIRQ_Attended()
%------------------------------------------------------------------------------
 %- Do nothing for bareboard
%SUBROUTINE_END
%-
%-
%------------------------------------------------------------------------------
%SUBROUTINE RTOSAdap_genRTOSReturnIRQ_NOTAttended()
%------------------------------------------------------------------------------
 %- Do nothing for bareboard
%SUBROUTINE_END
%-
